#!/usr/bin/python
# encoding: utf-8

########################################################################
# Description: hal_ajog_bbb.py                                         #
# This code reads an ADC input on the BeagleBone and converts the      #
# resulting value into a new value witch can be use for analog jog     #
#                                                                      #
# Author(s): Charles Steinkuehler (input algorithm from hal_temp_bbb)  #
#            Jean-Luc Soumard     (new input/output for analog jog)    #
# License: GNU GPL Version 2.0 or (at your option) any later version.  #
#                                                                      #
# Major Changes:                                                       #
# 2016-August Jean-Luc Soumard                                         #
#             Initial version for analog jog                           #
########################################################################
#                                                                      #
# This program is free software; you can redistribute it and/or        #
# modify it under the terms of the GNU General Public License          #
# as published by the Free Software Foundation; either version 2       #
# of the License, or (at your option) any later version.               #
#                                                                      #
# This program is distributed in the hope that it will be useful,      #
# but WITHOUT ANY WARRANTY; without even the implied warranty of       #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        #
# GNU General Public License for more details.                         #
#                                                                      #
# You should have received a copy of the GNU General Public License    #
# along with this program; if not, write to the Free Software          #
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA        #
# 02110-1301, USA.                                                     #
#                                                                      #
# THE AUTHORS OF THIS PROGRAM ACCEPT ABSOLUTELY NO LIABILITY FOR       #
# ANY HARM OR LOSS RESULTING FROM ITS USE.  IT IS _EXTREMELY_ UNWISE   #
# TO RELY ON SOFTWARE ALONE FOR SAFETY.  Any machinery capable of      #
# harming persons must have provisions for completely removing power   #
# from all motors, etc, before persons enter any danger area.  All     #
# machinery must be designed to comply with local and national safety  #
# codes, and the authors of this software can not, and do not, take    #
# any responsibility for such compliance.                              #
########################################################################

########################################################################
# How to use
# 
# loadusr -Wn AinJog ./hal_ain_bbb.py -n AinJog -c 04:50:300,05:50:300 -o AJOG
#
# halpin.value returns different values depending on "param"
# param = VOLTS, RANGE, AJOG 
# arguments = chanel:param1:param2     example 04:50:300 for ch 4, max 300mm/mn, dead 50mm/mn with AJOG
#                                              04:50:300 for ch 4, max 300mm/mn, min 50mm/mn with RANGE
#                                              04:00:00  for ch 4, param not use by VOLTS which returns 0-1.8 value
# halpin.raw returns a value between 0 and 4095 which is direct 12 bits AN converter
########################################################################

import argparse
import glob
import sys
import time

import hal

# transforme la lecture 0-4095 en volts	
def adc2volts(pin):
    #print("sub-VOLTS")
    V_adc = pin.rawValue * 1.8 / 4096.0
    R_T = V_adc
    return R_T

# transforme la lecture 0-4095 en valeur comprise dans la plage param1 à param2
def adc2range(pin):
    #print("sub-RANGE")
    V_adc = pin.param1 + pin.rawValue * (pin.param2 - pin.param1) / 4096.0
    R_T = V_adc
    return R_T

# transforme la lecture 0-4095 en valeur param2/0/param2 avec une zone de neutre de param1
def adc2ajog(pin):
    #print("sub-AJOG")
    V_adc = pin.rawValue
    V_max = pin.param2
    V_dead = pin.param1
    pin.rawMinus = False
    pin.rawPlus = False
    if(V_adc > 2048):
        R_T = (V_adc - 2048) * V_max / 2048.0
        pin.rawMinus = False
        pin.rawPlus = True
    else:
        R_T = (2048 - V_adc) * V_max / 2048.0
        pin.rawMinus = True
        pin.rawPlus = False
    if(R_T < V_dead):
        R_T = 0.0
        pin.rawMinus = False
        pin.rawPlus = False
    return R_T
	

# déclaration de pin	
class Pin:
    def __init__(self):
        self.pin = 0
        self.r2temp = None
        self.halValuePin = 0
        self.halRawPin = 0
        self.filterSamples = []
        self.filterSize = 10
        self.rawValue = 0.0
        self.filename = ""
        self.filterSamples = []
        self.rawValue = 0.0
        self.rawMinus = False
        self.rawPlus = False
        self.halMinusPin = False
        self.halPlusPin = False
        self.param1 = 0.0
        self.param2 = 0.0

    def addSample(self, value):
        self.filterSamples.append(value)
        if (len(self.filterSamples) > self.filterSize):
            self.filterSamples.pop(0)
        sampleSum = 0.0
        for sample in self.filterSamples:
            sampleSum += sample
        self.rawValue = sampleSum / len(self.filterSamples)

# traite la lecture en fonction de l'option de sortie
def adc2OutPut(pin):
    #print(args.output)
    if(args.output == 'AJOG'):
        R = adc2ajog(pin)
    elif (args.output == 'VOLTS'):
        R = adc2volts(pin)
    elif (args.output == 'RANGE'):
        R = adc2range(pin)
    else:
        print("Invalid -o output type : %s" % args.output)
        print("Valid outputs are: AJOG VOLTS RANGE")
        sys.exit(1)
    return round(R * 1000.0) / 1000.0

# crée le nom de pin au format voulu
def getHalName(pin):
    return "ch-" + '{0:02d}'.format(pin.pin)

# vérifie si tout est prêt pour lire les entrées analogiques
def checkAdcInput(pin):
    syspath = '/sys/devices/ocp.*/44e0d000.tscadc/tiadc/iio:device0/'
    tempName = glob.glob(syspath + 'in_voltage' + str(pin.pin) + '_raw')
    #print (tempName)
    pin.filename = tempName[0]
    #print ("suite...")
    try:
        if len(pin.filename) > 0:
            f = open(pin.filename, 'r')
            f.close()
            time.sleep(0.001)
        else:
            raise UserWarning('Bad Filename')
    except (UserWarning, IOError):
        print(("Cannot read ADC input: %s" % pin.filename))
        sys.exit(1)


parser = argparse.ArgumentParser(description='HAL component to read ADC values and convert to user value')
parser.add_argument('-n','--name', help='HAL component name',required=True)
parser.add_argument('-i', '--interval', help='Adc update interval', default=0.05)
parser.add_argument('-c', '--channels', help='Komma separated list of channels and thermistors to use e.g. 01:semitec_103GT_2,02:epcos_B57560G1104', required=True)
parser.add_argument('-f', '--filter_size', help='Size of the low pass filter to use', default=10)
parser.add_argument('-o', '--output', help='Type of user output', default='VOLTS')

args = parser.parse_args()

updateInterval = float(args.interval)
filterSize = int(args.filter_size)
error = False
watchdog = True

# Create pins ----------------------------------------------------
pins = []

if (args.channels != ""):                       # il y a des arguments de type channels
    channelsRaw = args.channels.split(',')      # ils sont séparés par des virgules
    for channel in channelsRaw:                 # chaque channel regroupe ses arguments
        pinRaw = channel.split(':')             #   qui sont séparés par des :
        #print (len(pinRaw))
        if (len(pinRaw) != 3):                  #   le premier représente le n° d'entrée et est sur 2 caractères sinon erreur
            print(("wrong input"))
            sys.exit(1)
            
        pin = Pin()
        pin.pin = int(pinRaw[0])                # le n° de AIN est dans pinRaw[0]
        if ((pin.pin > 6) or (pin.pin < 0)):    # et doit être entre 0 et 5 (moi j'aurai bien mis 6 à la place de 5)
            print(("Pin not available"))
            sys.exit(1)
        checkAdcInput(pin)                      # pour vérifier si tout est en ordre au niveau des fichiers
        
        if (pinRaw[1] != ""):                   # le premier paramètre pour la valeur MINI ou DEAD
            pin.param1 = float(pinRaw[1])
        else:
            pin.param1 = 0.0
            
        if (pinRaw[2] != ""):                   # le deuxieme parametre pour la valeur MAXI
            pin.param2 = float(pinRaw[2])       
        else:
            pin.param2 = 4095.0
        pin.filterSize = filterSize
        pins.append(pin)

# Initialize HAL --------------------------------------------------------------------------
h = hal.component(args.name)
for pin in pins:
    pin.halRawPin = h.newpin(getHalName(pin) + ".raw", hal.HAL_FLOAT, hal.HAL_OUT)
    pin.halMinusPin = h.newpin(getHalName(pin) + ".minus", hal.HAL_BIT, hal.HAL_OUT)
    pin.halPlusPin = h.newpin(getHalName(pin) + ".plus", hal.HAL_BIT, hal.HAL_OUT)
    pin.halValuePin = h.newpin(getHalName(pin) + ".value", hal.HAL_FLOAT, hal.HAL_OUT)
halErrorPin = h.newpin("error", hal.HAL_BIT, hal.HAL_OUT)
halNoErrorPin = h.newpin("no-error", hal.HAL_BIT, hal.HAL_OUT)
halWatchdogPin = h.newpin("watchdog", hal.HAL_BIT, hal.HAL_OUT)
h.ready()

halErrorPin.value = error
halNoErrorPin.value = not error
halWatchdogPin.value = watchdog

try:
    while (True):
        try:
            for pin in pins:
                f = open(pin.filename, 'r')
                value = float(f.readline())
                pin.addSample(value)
                pin.halRawPin.value = pin.rawValue
                pin.halMinusPin.value = pin.rawMinus
                pin.halPlusPin.value = pin.rawPlus
                pin.halValuePin.value = adc2OutPut(pin)
            error = False
        except IOError:
            error = True

        halErrorPin.value = error
        halNoErrorPin.value = not error
        watchdog = not watchdog
        halWatchdogPin.value = watchdog
        time.sleep(updateInterval)
except:
    print(("exiting HAL component " + args.name))
    h.exit()

